# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

import asynchat
import asyncore
import socket
import threading
import PGUtils
# get built-in json module in 2.6, or simplejson if available, or fake json object
json = PGUtils.import_json()

LENGTH_TERMINATOR = ':'

class PGChat (asynchat.async_chat):
    def __init__(self, conn, controller=None):
        asynchat.async_chat.__init__(self, conn)
        self.controller = controller
        self.ibuffer = []
        self.set_terminator(LENGTH_TERMINATOR)
        # stores player username sent over the connection
        self.player_name = None
        
    # async_chat methods
    def collect_incoming_data(self, data):
        self.ibuffer.append(data)
        
    def found_terminator(self):
        msg = ''.join(self.ibuffer)
        self.ibuffer = []
        if self.get_terminator()==LENGTH_TERMINATOR:
            length = int(msg.strip())
            self.set_terminator(length)
        else:
            self.handle_message(msg)
            self.set_terminator(LENGTH_TERMINATOR)
            
    def handle_message(self, msg):
        #print 'got message:',msg
        obj = self.parse_object(msg)
        if self.controller: 
            self.controller.handle_network_message(self, obj)
        
    # custom methods
    def run(self):
        def run_in_thread():
            asyncore.loop()
            
        t = threading.Thread(target=run_in_thread)
        t.setDaemon(True)
        t.start()
    
    # handling dropped connections is screwy, if we don't do this we get stuck in an infinite loop
    # push(None) triggers an exception in send_object somehow, which stops the loop
    def handle_error(self):
        import traceback
        traceback.print_exc()
        print 'Connection dropped'
        self.push(None)
        self.close()
                
    # from http://mail.python.org/pipermail/spambayes-checkins/2003-January/000803.html
    # Python 2.6 removes ac_out_buffer from asynchat, so check with getattr
    def flush(self):
        """ Flush everything in the output buffer. """
        while self.producer_fifo or getattr(self, 'ac_out_buffer', None):
            self.initiate_send()

    def send_object(self, obj):
        objstr = json.dumps(obj)
        msg = str(len(objstr))+':'+objstr
        try:
            self.push(msg)
            # if there was a disconnect, self.connected will be set to False
            # asynchat apparently doesn't automatically flush, we need to make sure the full message is sent
            disconnect = not self.connected
            if not disconnect:
                self.flush()
        except Exception,ex:
            print 'exception in send_object',ex
            disconnect = True
        if disconnect:
            #print "Disconnecting..."
            self.controller.handle_network_disconnect()
        
    def parse_object(self, msg):
        """ Convert lists in decoded object to tuples so they can be hashed """
        return PGUtils.lists_to_tuples(json.loads(msg))
        
# testing
chat = [None]

def set_chat(c):
    global chat
    print 'setting chat',c
    chat[0] = c
    print 'chat=',chat[0]

def listen_for_connection_in_thread(port):
    def accept_connection():
        cs,addr = ss.accept()
        print 'Entering loop'
        set_chat(PGChat(cs))
        asyncore.loop()
        print 'Done with loop'  
              
    ss = socket.socket()
    ss.bind(('',port))
    ss.listen(1)
    t = threading.Thread(target=accept_connection)
    t.setDaemon(True)
    t.start()
    return ss
        
if __name__=='__main__':
    ss = socket.socket()
    ss.bind(('', 8888))
    ss.listen(1)
    cs,addr = ss.accept()
    chat = PGChat(cs)
    print 'Entering loop'
    asyncore.loop()
    print 'Done with loop'